/***************************************************************************
 *   Copyright (C) 2005-2006 by Konrad Wilhelm Kleine
 *   konrad@plothe-kleine.de
 *
 *   This file contains the implementation of globally used types, functions
 *   and constants for the Linux Printer Setup project.
 *
 *   PLEASE NOTICE: If not stated otherwise, whenever we speak of 'printer
 *                  drivers' or simply 'drivers', we mean the PPD files
 *                  and not the actual drivers.
 *
 *   STATUS: File has been reviewed and commented
 *
 *   $Id: globalsprintersetup.cpp,v 1.5 2006/04/23 16:06:22 hbci Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software Foundation,
 *   Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ***************************************************************************/

// App includes
#include "globalsprintersetup.h"

// MoneyPenny includes
#include "globals.h"

// Qt includes
#include <qstring.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qlistbox.h>
#include <qobject.h>
#include <qcursor.h>
#include <qinputdialog.h>

// Other includes
#include <stdlib.h>
using namespace std;

namespace LinuxPrinterSetup
{


/**
 * Constructs a printer device object without setting any members.
 */
PrinterDevice::PrinterDevice()
{
}


/**
 * Constructs a printer device object and sets the members.
 *
 * @param nativeManufacturer
 * @param nativeProduct
 * @param connectionType
 * @param drivers
 */
PrinterDevice::PrinterDevice( const QString & nativeManufacturer,
                              const QString & nativeProduct,
                              ConnectionType connectionType,
                              const QDomNodeList & drivers)
{
        this->nativeManufacturer  = nativeManufacturer;
        this->nativeProduct       = nativeProduct;
        this->connectionType      = connectionType;
        this->drivers             = drivers;
}


/**
 * Constructs a LPS notice object and sets the message to \a str .
 *
 * @param str
 */
LpsNotice::LpsNotice( const char * str ) : QString(str)
{
}


/**
 * Same as above, except for the type of the \a str parameter.
 * Constructs a LPS notice object and sets the message to \a str .
 *
 * @param str
 */
LpsNotice::LpsNotice( const QString & str ) : QString(str)
{
}


/**
 * Constructs a LPS error object and sets the message to \a str .
 *
 * @param str
 */
LpsError::LpsError( const char * str ) : QString(str)
{
}


/**
 * Same as above, error for the type of the \a str parameter.
 * Constructs a LPS notice object and sets the message to \a str .
 *
 * @param str
 */
LpsError::LpsError( const QString & str ) : QString(str)
{
}


/**
 * The MoneyPenny linux live CD uses Knoppix where you can use \c sudo
 * on shell level WITHOUT entering a password. In contrast to this,
 * on other non-live-CD linux systems, you MUST enter a password.
 * This function offers a small workaround for the \c sudo command.
 * If you want to execute a system command with root rights, you can
 * prepend the string returned from this function to the system command.
 * The function automatically determines if you are working on a live CD
 * or not and pops up a password input dialog if needed.
 *
 * @returns  system dependend sudo string
 */
QString sudo()
{
  bool    okPressed;

  QString result      = "sudo ";

  if ( !USE_LIVE_CD_FEATURES )
  {
    //
    // Since we are not operating under Knoppix, we ask the user for the root password
    //
    QString rootPassword =  QInputDialog::getText(
                              QObject::tr("Information"),
                              QObject::tr("Please enter the root password to execute the printer setup command."),
                              QLineEdit::Password,
                              0,
                              &okPressed
                            );

    if ( okPressed )
    {
      result  = QString("echo ") + rootPassword + " | sudo -S ";
    }
  }

  return result;
}


//
//====================================================================================
//
// Shared functions
//
// The following functions are used by both printer setups, the automatic and manual.
// Therefore, they are outsourced from the concrete classes and made friends.
//
//====================================================================================
//


/**
 * This function tries to find the appropriate XML information file for the
 * \a driverName in the filesystem. If the file exists, it's information is
 * extracted and presentated in a message box.
 * Such information includes detailed information of the driver capabilities.
 *
 * @see     LPS_DRIVER_DETAILS_DIR
 * @param   driverName
 * @throws  LpsError                if \c xsltproc command fails
 */
void showPrinterDriverDetails( const QString & driverName )
{
  QString       xmlFileName = driverName.section("/", 1, 1);
  FILE        * pipe        = NULL;
  QTextStream * reader      = NULL;

  try
  {
    ///
    /// This is how we present the XML driver information file:
    /// We pass parameters to a system call to \c xsltproc which does the transformation of the
    /// XML file into a displayable string.
    ///

    QString xmlFilePath = QString(LPS_DRIVER_DETAILS_DIR + xmlFileName + ".xml").remove(" (recommended)");
    QString params      = "--nonet --stringparam commentTitle " + QObject::tr("Comment");
    QString cmd         = "xsltproc "+params+" "+MP_DIR() + MONEYPENNY_DATA_DIR + LPS_DRIVER_STYLESHEET_PATH+" " + xmlFilePath;
    pipe                = popen(cmd.latin1(), "r");

    if ( !pipe )
    {
      throw "Error during xsltproc execution! Could not execute process:<br />" + cmd;
    }

    reader          = new QTextStream(pipe, IO_ReadOnly);

    QString details = reader->read().replace("&gt;", ">").replace("&lt;", "<");

    QMessageBox::information(0, QObject::tr("Driver Details"), details);
  }
  catch ( LpsError e )
  {
    //
    // NEVER change the order of these deletions/closings!!!
    //
    if (reader)
      delete reader;
    if (pipe)
      pclose(pipe);

    throw e;
  }

  //
  // NEVER change the order of these deletions/closings!!!
  //
  if (reader)
    delete reader;
  if (pipe)
    pclose(pipe);
}


/**
 * This function pops up a yes/no message box, that asks if one wants to
 * print test page. If "yes" is chosen, the page is send to the printer
 * that's supposed to actually print it.
 * You must supply the \a widget from which this function is called,
 * because it's mouse cursor is set to a sandclock/wait symbol.
 *
 * @see                 LPS_TESTPAGE_PS_SRC
 * @param   widget      calling widget
 * @param   printerName the name of the printer on CUPS level
 * @throws  LpsError    if something \c lpr command fails
 */
void testPrintRequest( QWidget * widget, const QString & printerName )
{
  //
  // Ask the user if a test print is requested
  //
  QString question = QObject::tr("Your printer has been configured successfully.") + "<br>" +
                     QObject::tr("Would you like to print a testpage?");

  int answer = QMessageBox::question(widget,
                                     LPS_APP_NAME,
                                     question,
                                     QMessageBox::Yes,
                                     QMessageBox::No,
                                     QMessageBox::NoButton);

  if ( answer == QMessageBox::Yes )
  {
    //
    // Build the path to test print file
    //
    QString testFile = MP_DIR() + MONEYPENNY_DATA_DIR + "/" + LPS_TESTPAGE_PS_SRC;
    QString cmdLpr = "lpr -P "+printerName+" -C "LPS_TESTPAGE_JOBNAME" "+testFile;
    qDebug("Printing Testpage: %s", cmdLpr.latin1());

    //
    // Now print the test page.
    // Due to system() call, the UI is blocked.
    //
    widget->setCursor(Qt::WaitCursor);
    if ( system(cmdLpr) != 0 )
    {
      widget->unsetCursor();
      throw LpsError("Error during lpr execution! Could not execute process: " + cmdLpr);
    }
    widget->unsetCursor();

    //
    // Open preview window
    //
    QDialog dlg;
    dlg.setFixedSize(LPS_TESTPAGE_WIDTH, LPS_TESTPAGE_HEIGHT);
    QPixmap pxm(MP_DIR() + MONEYPENNY_DATA_DIR + "/" + LPS_TESTPAGE_IMG_SRC);
    pxm.resize(LPS_TESTPAGE_WIDTH, LPS_TESTPAGE_HEIGHT);
    dlg.setPaletteBackgroundPixmap(pxm);
    dlg.setCaption(QObject::tr("Testprint Preview"));
    dlg.exec();
  }
}


/**
 * This function installs a printer with the name \a printerName , connected
 * to the device \a deviceUri and the driver file \a ppdFilePath .
 *
 * @param   widget      calling widget
 * @param   printerName the name of the new printer on CUPS level
 * @param   deviceUri   the connection type already formatted for use with \c lpadmin
 * @param   ppdFilePath the file path to the driver file
 * @throws  LpsError    if something CUPS' \c lpadmin command fails
 */
void installPrinter( QWidget * widget,
                     const QString & printerName,
                     const QString & deviceUri,
                     const QString & ppdFilePath )
{
  //
  /// Build install command string and replace the blanks in \c m_printername with
  /// underscores, otherwise \c lpadmin will fail.
  /// @todo find out on which systems \c sudo() is really needed (SuSE is gay)
  //
  QString cmdLpadmin = sudo() + LPS_LPADMIN_PATH "lpadmin -p " + QString(printerName).replace(" ", "_") + " -E -v " + deviceUri + " -m " + ppdFilePath + "";

  //
  // Install the printer.
  // Due to system() call, the UI is blocked.
  //
  widget->setCursor(Qt::WaitCursor);
  if (  system(cmdLpadmin) != 0 )
  {
    widget->unsetCursor();
    qDebug("GuiPrinterManualSetup::on_pushButton_saveSettings_clicked: Failed command: %s", cmdLpadmin.latin1());
    throw LpsError("Error during lpadmin execution! Could not execute process: " + cmdLpadmin);
  }

  system("sync");

  widget->unsetCursor();
}

};
